/*
 * include/asm-arm/arch-cxd4132/platform.h
 *
 * CXD4132 address map
 *
 * Copyright 2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __ARCH_CXD4132_PLATFORM_H
#define __ARCH_CXD4132_PLATFORM_H

#ifdef CONFIG_CXD4132_DDR_RANK0_ABSENT
#define DDR_BASE		0x90000000
#define DDR_SIZE		SZ_256M
#else
#define DDR_BASE		0x80000000
#define DDR_SIZE		SZ_512M
#endif

/*---------------- FSP ----------------*/
#define FSP_CS0_BASE		0x00000000
#define FSP_CS1_BASE		0xb0000000
#define FSP_CS1_SIZE		(SZ_64M)
#define FSP_CS2_BASE		0xb8000000
#define FSP_CS2_SIZE		(SZ_64M)

/*---------------- AHB ----------------*/
#define CXD4132_SRAM_BASE	0xa0000000
#define VA_BOOTRAM		IO_ADDRESS(CXD4132_SRAM_BASE)
#define VA_BOOTRAM_CACHED	0xe0000000

/*---------------- AHB ----------------*/
#define CXD4132_AHB_BASE	0xf0000000
#define CXD4132_AHB_SIZE	(SZ_16M)

#define CXD4132_DMAC0_BASE	(CXD4132_AHB_BASE)
#define CXD4132_DMAC0_SIZE	(SZ_4K)

/*---------------- APB ----------------*/
#define CXD4132_APB_BASE	0xf2000000
#define CXD4132_APB_SIZE	(SZ_64M)

#define CXD4132_APB0_BASE	(CXD4132_APB_BASE + 0x0000000)
/* FSP */
#define CXD4132_FSP_BASE	(CXD4132_APB0_BASE + 0x000000)
#define CXD4132_FSP_SIZE	(SZ_1K * 20)
#define VA_FSP			IO_ADDRESSP(CXD4132_FSP_BASE)
/* TIMER */
#define CXD4115_TIMER_BASE(x)	(CXD4132_APB0_BASE + 0x008000 + 0x20*(x))
#define VA_TIMER(x)		IO_ADDRESSP(CXD4115_TIMER_BASE(x))
#define CXD4132_WDT		12
/* SIO */
#define CXD4132_SIO(x)		(CXD4132_APB0_BASE + 0x010000 + 0x200*(x))
/* UART */
#define CXD4115_UART(x)		(CXD4132_APB0_BASE + 0x038000 + 0x1000*(x))
/* PER_MISC */
#define CXD4132_PERMISC_BASE	(CXD4132_APB0_BASE + 0x068000)
#define VA_PERMISC		IO_ADDRESSP(CXD4132_PERMISC_BASE)

#define CXD4132_APB1_BASE	(CXD4132_APB_BASE + 0x1000000)
/* GPIO */
#define CXD4132_GPIO(x)		(CXD4132_APB1_BASE + 0x000000 + 0x100*(x))
#define PA_GPIO(x)		CXD4132_GPIO(x)
#define VA_GPIO(x)		IO_ADDRESSP(CXD4132_GPIO(x))
/* OCTRL */
#define CXD4132_OCTRL(x)	(CXD4132_APB1_BASE + 0x010000 + 0x100*(x))
#define VA_OCTRL(x)		IO_ADDRESSP(CXD4132_OCTRL(x))
/* CLKRST */
#define CXD4132_CLKRST(x)	(CXD4132_APB1_BASE + 0x020000 + 0x10*(x))
#define VA_CLKRST(x)		IO_ADDRESSP(CXD4132_CLKRST(x))
/* MISC */
#define CXD4132_MISCCTRL(x)	(CXD4132_APB1_BASE + 0x060000 + 0x10*(x))
#define VA_MISCCTRL(x)		IO_ADDRESSP(CXD4132_MISCCTRL(x))
/* SDC */
#define CXD4132_AVB_BASE	(CXD4132_APB1_BASE + 0x00800000)
#define CXD4132_AVBCKG_BASE	(CXD4132_AVB_BASE + 0x0000)
#define VA_AVBCKG		IO_ADDRESSP(CXD4132_AVBCKG_BASE)
#define CXD4132_BAM_BASE	(CXD4132_AVB_BASE + 0x1000)
#define CXD4132_BAM_SIZE	(SZ_4K)
#define VA_BAM			IO_ADDRESSP(CXD4132_BAM_BASE)
#define CXD4132_SDC_BASE	(CXD4132_AVB_BASE + 0x3000)
#define CXD4132_SDC_SIZE	(SZ_64K)
#define VA_SDC			IO_ADDRESSP(CXD4132_SDC_BASE)

#define CXD4132_APB2_BASE	(CXD4132_APB_BASE + 0x2000000)

#define CXD4132_APB3_BASE	(CXD4132_APB_BASE + 0x3000000)
/* HTP */
#define CXD4132_HTP_BASE	(CXD4132_APB3_BASE + 0x000000)

/*---------------- BOOT ----------------*/
#define CXD4132_HIO_BASE	0xf8000000
#define CXD4132_HIO_SIZE	(SZ_1M)
/* MPCore */
#define CXD4132_MPCORE_BASE	(CXD4132_HIO_BASE+0x00000)
#define CXD4132_SCU_BASE	(CXD4132_MPCORE_BASE+0x0000)
#define CXD4132_GIC_CPU_BASE	(CXD4132_MPCORE_BASE+0x0100) /* CPU I/F */
#define CXD4132_LOCALTIMER_BASE	(CXD4132_MPCORE_BASE+0x0600)
#define CXD4132_LOCALTIMER(x)	(CXD4132_MPCORE_BASE+0x0700+0x100*(x))
#define CXD4132_GIC_DIST_BASE	(CXD4132_MPCORE_BASE+0x1000) /* INT DIST */
#define CXD4132_LOCALTIMER_BASE_SIZE (0x0100)
#define SCU_BASE		CXD4132_SCU_BASE
#define VA_SCU			IO_ADDRESSP(CXD4132_SCU_BASE)
#define VA_GIC_CPU		IO_ADDRESSP(CXD4132_GIC_CPU_BASE)
#define VA_LOCALTIMER		IO_ADDRESSP(CXD4132_LOCALTIMER_BASE)
#define VA_LOCALTIMER_X(x)	HIO_ADDRESSP(CXD4132_LOCALTIMER(x))
#define VA_GIC_DIST		IO_ADDRESSP(CXD4132_GIC_DIST_BASE)
#define GIC_CPU_BASE		CXD4132_GIC_CPU_BASE
/* L2 Cache */
#define CXD4132_PL310_BASE	(CXD4132_HIO_BASE+0x02000)
#define VA_PL310		IO_ADDRESSP(CXD4132_PL310_BASE)

#ifndef __ASSEMBLY__
extern int cxd4132_core_init(void);
extern struct sys_timer cxd4115_timer;
extern void cxd4132_map_io(void);
extern void cxd4132_init_irq(void);
extern int cxd41xx_smode;
#endif /* !__ASSEMBLY__ */

#endif /* __ARCH_CXD4132_PLATFORM_H */
