/*
 * include/asm-arm/arch-cxd4132/regs-perimisc.h
 *
 * PERIMISC registers
 *
 * Copyright 2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __REGS_PERIMISC_H__
#define __REGS_PERIMISC_H__

/*----- PER_RSTREQ -----*/
#define PER_RSTREQ	0x0000
# define PER_RSTREQ_CPUCLR	0x100
# define PER_RSTREQ_EN		0x010
# define PER_RSTREQ_ASSERT	0x001
#define PER_RSTREQ_SET  0x0004
#define PER_RSTREQ_CLR  0x0008
#define PER_RSTREQ_STAT 0x000c
# define PER_RSTREQ_ST_TIM	0x10
# define PER_RSTREQ_ST_CPU2	0x04
# define PER_RSTREQ_ST_CPU1	0x02
# define PER_RSTREQ_ST_CPU0	0x01

#define VA_PER_RSTREQ		(VA_PERMISC + PER_RSTREQ)
#define VA_PER_RSTREQ_SET	(VA_PERMISC + PER_RSTREQ_SET)
#define VA_PER_RSTREQ_CLR	(VA_PERMISC + PER_RSTREQ_CLR)
#define VA_PER_RSTREQ_STAT	(VA_PERMISC + PER_RSTREQ_STAT)

/*----- PER_FIQEN -----*/
#define PER_FIQEN	0x0010
# define PER_FIQEN_BM1		0x00000000
# define PER_FIQEN_BM2		0x00000010
# define PER_FIQEN_ALL		0x00000007
#define VA_PER_FIQEN		(VA_PERMISC + PER_FIQEN)

/*----- PER_CLKRST -----*/
#define PER_CLKRST	0x1000
# define PER_CLKRST_SET		0x04
# define PER_CLKRST_CLR		0x08
# define PER_CLKRST_STAT	0x0c

# define PER_CLKRST_SIO		0x0380
#  define PER_CLKRST_SIO_PCLK	0x00
#  define PER_CLKRST_SIO_CLK	0x10

# define PER_CLKRST_UART	0x0430
#  define PER_CLKRST_UART_PCLK	0x00
#  define PER_CLKRST_UART_CLK	0x10

# define PER_CLKRST_SRST_UART	0x11b0

# define PER_CLKRST_AUTOPS_C	0x1310
#  define PER_CLKRST_AUTOPS_UART0	0x01000000

#define CXD4132_SRST_WAIT 1000

#define CXD4132_PER_CLKRST	(CXD4132_PERMISC_BASE + PER_CLKRST)

#define CXD4132_PER_CLKRST_SIO(x) (CXD4132_PER_CLKRST+PER_CLKRST_SIO+0x20*(x))
#define CXD4132_PER_CLKRST_SIO_PCLK(x)	(CXD4132_PER_CLKRST_SIO(x) + PER_CLKRST_SIO_PCLK)
#define CXD4132_PER_CLKRST_SIO_CLK(x)	(CXD4132_PER_CLKRST_SIO(x) + PER_CLKRST_SIO_CLK)

#define CXD4132_PER_CLKRST_UART(x) (CXD4132_PER_CLKRST + PER_CLKRST_UART + 0x20*(x))
#define CXD4132_PER_CLKRST_UART_PCLK(x)	(CXD4132_PER_CLKRST_UART(x) + PER_CLKRST_UART_PCLK)
#define CXD4132_PER_CLKRST_UART_CLK(x)	(CXD4132_PER_CLKRST_UART(x) + PER_CLKRST_UART_CLK)

#define VA_PER_CLKRST		IO_ADDRESSP(CXD4132_PER_CLKRST)
#define VA_PER_CLKRST_UART(x)	IO_ADDRESSP(CXD4132_PER_CLKRST_UART(x))
#define VA_PER_CLKRST_UART_PCLK(x) IO_ADDRESSP(CXD4132_PER_CLKRST_UART_PCLK(x))
#define VA_PER_CLKRST_UART_CLK(x) IO_ADDRESSP(CXD4132_PER_CLKRST_UART_CLK(x))

#define CXD4132_PER_CLKRST_SRST_UART(x) (CXD4132_PER_CLKRST + PER_CLKRST_SRST_UART + 0x10*(x))
#define CXD4132_PER_CLKRST_SRST_UART_STAT(x)	(CXD4132_PER_CLKRST_SRST_UART(x) + PER_CLKRST_STAT)
#define VA_PER_CLKRST_SRST_UART(x) IO_ADDRESSP(CXD4132_PER_CLKRST_SRST_UART(x))
#define VA_PER_CLKRST_SRST_UART_STAT(x) IO_ADDRESSP(CXD4132_PER_CLKRST_SRST_UART_STAT(x))

#endif /* __REGS_PERIMISC_H__ */
